package de.ugoe.cs.quest.ui.commands;

import java.security.InvalidParameterException;
import java.util.Collection;
import java.util.List;
import java.util.Map.Entry;
import java.util.SortedMap;
import java.util.TreeMap;

import de.ugoe.cs.quest.CommandHelpers;
import de.ugoe.cs.quest.SequenceInstanceOf;
import de.ugoe.cs.quest.eventcore.Event;
import de.ugoe.cs.util.console.Command;
import de.ugoe.cs.util.console.Console;
import de.ugoe.cs.util.console.GlobalDataContainer;

/**
 * <p>
 * Command to print basic statistical information about stored sequences, e.g.,
 * how many there are of which lenght.
 * </p>
 * 
 * @author Steffen Herbold
 * @version 1.0
 */
public class CMDsequenceStatistics implements Command {

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.ugoe.cs.util.console.Command#run(java.util.List)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void run(List<Object> parameters) {
		String sequencesName;
		try {
			sequencesName = (String) parameters.get(0);
		} catch (Exception e) {
			throw new InvalidParameterException();
		}

		Collection<List<Event>> sequences = null;
		Object dataObject = GlobalDataContainer.getInstance().getData(
				sequencesName);
		if (dataObject == null) {
			CommandHelpers.objectNotFoundMessage(sequencesName);
			return;
		}
		if (!SequenceInstanceOf.isCollectionOfSequences(dataObject)) {
			CommandHelpers.objectNotType(sequencesName,
					"Collection<List<Event<?>>>");
			return;
		}

		sequences = (Collection<List<Event>>) dataObject;
		Console.println("Number of Sequences: " + sequences.size());
		SortedMap<Integer, Integer> lengthMap = new TreeMap<Integer, Integer>();
		for (List<Event> sequence : sequences) {
			Integer currentSize = sequence.size();
			if (lengthMap.containsKey(currentSize)) {
				lengthMap.put(currentSize, lengthMap.get(currentSize) + 1);
			} else {
				lengthMap.put(currentSize, 1);
			}
		}
		for (Entry<Integer, Integer> entry : lengthMap.entrySet()) {
			Console.println("Of length " + entry.getKey() + ": "
					+ entry.getValue());
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.ugoe.cs.util.console.Command#help()
	 */
	@Override
	public String help() {
		return "sequenceStatistics <sequencesName>";
	}

}
